'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var redux = require('redux');
var createSelector = require('create-selector');
var createRouteMatcher = _interopDefault(require('feather-route-matcher'));
var reselect = require('reselect');
var idbKeyval = _interopDefault(require('idb-keyval'));
var ric = _interopDefault(require('ric-shim'));
var debounce = _interopDefault(require('lodash/debounce'));
var URL = _interopDefault(require('url-parse'));
var qs = _interopDefault(require('querystringify'));

var appTimeBundle$1 = {
  name: 'appTime',
  reducer: Date.now,
  selectAppTime: function (state) { return state.appTime; }
}

var changes = {
  'START': 1,
  'SUCCESS': -1,
  'ERROR': -1
}

var asyncCount = {
  name: 'asyncCount',
  reducer: function (state, ref) {
    if ( state === void 0 ) state = 0;
    var type = ref.type;

    var last = type.split('_').slice(-1)[0]
    return state + (changes[last] || 0)
  },
  selectAsyncActive: function (state) { return state.asyncCount > 0; }
}

var debug = false
try { debug = !!window.localStorage.debug } catch (e) {}
var IS_BROWSER = typeof window !== 'undefined' || typeof self !== 'undefined'
var IS_DEBUG = debug
var flattenExtractedToObject = function (extracted) {
  var result = {}
  for (var appName in extracted) {
    extracted[appName].forEach(function (obj) { return Object.assign(result, obj); })
  }
  return result
}
var flattenExtractedToArray = function (extracted) {
  var accum = []
  for (var appName in extracted) {
    accum.push(extracted[appName])
  }
  return (ref = []).concat.apply(ref, accum)
  var ref;
}

var debugMiddleware = function (store) { return function (next) { return function (action) {
  if (IS_DEBUG) {
    console.group(action.type)
    console.info('action:', action)
  }

  var result = next(action)

  if (IS_DEBUG) {
    console.debug('state:', store.getState())
    window.listSelectors && window.listSelectors()
    window.listActiveEffects && window.listActiveEffects()
    console.groupEnd(action.type)
  }

  return result
}; }; }

function createThunkMiddleware (extra) {
  return function (ref) {
    var dispatch = ref.dispatch;
    var getState = ref.getState;

    return function (next) { return function (action) {
    if (typeof action === 'function') {
      return action(Object.assign({dispatch: dispatch, getState: getState}, extra))
    }
    return next(action)
  }; };
  }
}

var thunk = createThunkMiddleware()
thunk.withExtraArgs = createThunkMiddleware

var consumeBundle = function (bundle, accum, bundles) {
  if ( accum === void 0 ) accum = {};

  // simple test to see whether we have needed props
  if (!accum.reducers) {
    Object.assign(accum, {
      reducers: {},
      extraMiddleware: {},
      initMethods: {},
      extraArgs: {},
      itemsToExtract: {},
      actionCreators: {},
      selectors: {}
    })
  }
  var name = bundle.name
  Object.keys(bundle).forEach(function (key) {
    var value = bundle[key]
    if (key === 'reducer') {
      accum.reducers[name] = value
      return
    }
    if (key === 'getReducer') {
      accum.reducers[name] = value()
      return
    }
    if (key === 'getMiddleware') {
      accum.extraMiddleware[name] = value
      return
    }
    if (key === 'init') {
      accum.initMethods[name] = value
      return
    }
    if (key === 'extraArgs') {
      Object.assign(accum.extraArgs, value)
      return
    }
    if (key === 'extract') {
      accum.itemsToExtract[name] = bundles.reduce(function (accum, bundle) {
        var extracted = bundle[value]
        if (extracted) {
          accum[bundle.name] || (accum[bundle.name] = [])
          accum[bundle.name].push(extracted)
        }
        return accum
      }, {})
    }
    if (key.indexOf('do') === 0) {
      var obj = {}
      obj[key] = value
      Object.assign(accum.actionCreators, obj)
      return
    }
    if (key.indexOf('select') === 0) {
      var obj$1 = {}
      obj$1[key] = value
      Object.assign(accum.selectors, obj$1)
      return
    }
  })
  return accum
}

var bindSelectorsToStore = function (store, selectors) {
  var loop = function ( key ) {
    var selector = selectors[key]
    store[key] = function () { return selector(store.getState()); }
  };

  for (var key in selectors) loop( key );
}

var decorateStore = function (store, meta) {
  store.meta || (store.meta = {})
  var combinedSelectors = Object.assign(store.meta.selectors || {}, meta.selectors)
  createSelector.resolveSelectors(combinedSelectors)
  store.meta.selectors = combinedSelectors
  bindSelectorsToStore(store, combinedSelectors)
  Object.assign(store, redux.bindActionCreators(meta.actionCreators, store.dispatch))
  Object.assign(store.meta, meta)
  for (var appName in meta.initMethods) {
    meta.initMethods[appName](store, meta.itemsToExtract[appName])
  }
}

var composeBundles$1 = function () {
  var bundles = [], len = arguments.length;
  while ( len-- ) bundles[ len ] = arguments[ len ];

  // build out object of extracted bundle info
  var meta = {}
  bundles.forEach(function (bundle) { return consumeBundle(bundle, meta, bundles); })
  return function (data) {
    var middleware = [
      thunk.withExtraArgs(meta.extraArgs),
      debugMiddleware
    ]

    for (var appName in meta.extraMiddleware) {
      middleware.push(meta.extraMiddleware[appName](meta.itemsToExtract[appName]))
    }

    var store = redux.createStore(
      redux.combineReducers(meta.reducers),
      data,
      redux.applyMiddleware.apply(redux, middleware)
    )

    store.bundles = bundles

    decorateStore(store, meta)

    store.integrateBundles = function () {
      var bundlesToIntegrate = [], len = arguments.length;
      while ( len-- ) bundlesToIntegrate[ len ] = arguments[ len ];

      var bundles = store.bundles;
      var bundleMap = bundles.reduce(function (acc, bundle, index) {
        acc[bundle.name] = index
        return acc
      }, {})
      bundlesToIntegrate.forEach(function (newBundle) {
        var currentIndex = bundleMap[newBundle.name]
        if (currentIndex != null) {
          bundles[currentIndex] = newBundle
        } else {
          bundles.push(newBundle)
        }
      })
      var meta = {}
      bundles.forEach(function (bundle) { return consumeBundle(bundle, meta, bundles); })
      decorateStore(store, meta)
      store.replaceReducer(redux.combineReducers(meta.reducers))
    }

    return store
  }
}

var version = "6.1.0";

var debug$2
try {
  debug$2 = !!(window.localStorage.debug)
} catch (e) {}

var inspect = {
  name: 'inspect',
  init: function (store) {
    if (debug$2) {
      var names = store.bundles.map(function (bundle) { return bundle.name; })
      window.store = store
      var selectors = []
      var actionCreators = []
      for (var key in store) {
        var item = store[key]
        if (key.indexOf('select') === 0) {
          window[key] = item
          selectors.push(key)
        } else if (key.indexOf('do') === 0) {
          window[key] = item
          actionCreators.push(key)
        }
      }
      actionCreators.sort()
      selectors.sort()
      var colorTitle = 'color: #1676D2;'
      var black = 'color: black;'
      var colorGreen = 'color: #4CAF50;'
      var colorOrange = 'color: #F57C00;'
      var colorRed = 'color: #F44336;'
      var normal = 'font-weight: normal;'

      store.listSelectors = window.listSelectors = function () {
        var results = {}
        selectors.sort().forEach(function (selectorName) {
          results[selectorName] = store[selectorName]()
        })
        console.log('%cselectors:', colorGreen, results)
      }

      store.listBundles = window.listBundles = function () {
        console.log('%cinstalled bundles:\n  %c%s', colorTitle, black, names.join('\n  '))
      }

      store.listActionCreators = window.listActionCreators = function () {
        console.groupCollapsed('%caction creators', colorOrange)
        actionCreators.forEach(function (name) { return console.log(name); })
        console.groupEnd()
      }

      store.listEffects = window.listEffects = function () {
        var string = ''
        for (var selectorName in store.effects) {
          string += "\n  " + selectorName + " -> " + (store.effects[selectorName])
        }
        console.log('%ceffects:%c%s', colorOrange, black, string)
      }

      store.listActiveEffects = window.listActiveEffects = function () {
        var activeEffectQueue = store.activeEffectQueue;
        if (activeEffectQueue.length) {
          var selectorName = activeEffectQueue[0]
          var actionCreatorName = store.effects[selectorName]
          var result = store[selectorName]()
          console.log(
            ("%cnext effect:\n  %c%s() -> %c" + actionCreatorName + "(%c" + (JSON.stringify(result)) + "%c)"),
            colorOrange,
            black,
            selectorName,
            colorRed,
            black,
            colorRed
          )
          if (activeEffectQueue.length > 1) {
            console.log('%cqueued effects:', colorRed, activeEffectQueue.slice(1).join(', '))
          }
        }
      }

      console.groupCollapsed('%credux bundler v%s', colorTitle, version)
      store.listBundles()
      var exported = []
      for (var key$1 in store) {
        if (key$1.indexOf('select') === 0 || key$1.indexOf('do') === 0) {
          exported.push((key$1 + "()"))
        }
      }
      exported.sort()
      exported.unshift('store')
      console.log(("%cattached to window:\n  %c" + (exported.join('\n  '))), colorTitle, black + normal)
      store.listSelectors()
      store.listEffects()
      console.groupEnd()
      store.listActiveEffects()
    }
  }
}

var createRoutingBundle = function (routes) { return ({
  name: 'routes',
  selectRouteInfo: createSelector.createSelector('selectPathname', createRouteMatcher(routes)),
  selectRouteParams: createSelector.createSelector('selectRouteInfo', function (ref) {
    var params = ref.params;

    return params;
  }),
  selectCurrentComponent: createSelector.createSelector('selectRouteInfo', function (ref) {
    var page = ref.page;

    return page;
  })
}); }

var createTimeCheckSelector = function (timeSelector, age) { return reselect.createSelector(
  timeSelector,
  appTimeBundle$1.selectAppTime,
  function (time, appTime) {
    if (!time) {
      return false
    }
    var elapsed = appTime - time
    return elapsed > age
  }
); }

var defaultOpts = {
  name: null,
  getPromise: null,
  actionBaseType: null,
  staleAge: 900000, // fifteen minutes
  retryAfter: 60000, // one minute,
  persist: true
}

var createAsyncResourceBundle$1 = function (spec) {
  var opts = Object.assign({}, defaultOpts, spec)
  var name = opts.name;
  var staleAge = opts.staleAge;
  var retryAfter = opts.retryAfter;
  var actionBaseType = opts.actionBaseType;
  var ucaseName = name.charAt(0).toUpperCase() + name.slice(1)

  if (process.env.NODE_ENV !== 'production') {
    for (var item in opts) {
      if (opts[item] === null) {
        throw Error(("You must supply an " + item + " option when creating a resource bundle"))
      }
    }
  }

  // build selectors
  var inputSelector = function (state) { return state[name]; }
  var dataSelector = reselect.createSelector(
    inputSelector,
    function (resourceState) { return resourceState.data; }
  )
  var lastSuccessSelector = reselect.createSelector(inputSelector, function (resource) { return resource.lastSuccess; })
  var lastErrorSelector = reselect.createSelector(
    inputSelector,
    function (resource) { return resource.errorTimes.slice(-1)[0] || null; }
  )
  var isStaleSelector = createTimeCheckSelector(lastSuccessSelector, staleAge)
  var isWaitingToRetrySelector = createTimeCheckSelector(lastErrorSelector, retryAfter)
  var isLoadingSelector = reselect.createSelector(
    inputSelector,
    function (resourceState) { return resourceState.isLoading; }
  )
  var failedPermanentlySelector = reselect.createSelector(
    inputSelector,
    function (resourceState) { return resourceState.failedPermanantly; }
  )
  var shouldUpdateSelector = reselect.createSelector(
    isLoadingSelector,
    failedPermanentlySelector,
    isWaitingToRetrySelector,
    dataSelector,
    isStaleSelector,
    function (isLoading, failedPermanantly, isWaitingToRetry, data, isStale) {
      if (isLoading || failedPermanantly || isWaitingToRetry) {
        return false
      }
      if (!data) {
        return true
      }
      return isStale
    }
  )

  var actions = {
    START: (actionBaseType + "_START"),
    SUCCESS: (actionBaseType + "_SUCCESS"),
    ERROR: (actionBaseType + "_ERROR")
  }

  var initialState = {
    isLoading: false,
    data: null,
    errorTimes: [],
    lastSuccess: null,
    failedPermanantly: false
  }

  var doFetchData = function () { return function (args) {
    var dispatch = args.dispatch;
    dispatch({type: actions.START})
    delete args.dispatch
    return opts.getPromise(args)
      .then(function (payload) { return dispatch({type: actions.SUCCESS, payload: payload}); })
      .catch(function (error) { return dispatch({type: actions.ERROR, error: error}); })
  }; }

  var result = {
    name: name,
    reducer: function (state, ref) {
      if ( state === void 0 ) state = initialState;
      var type = ref.type;
      var payload = ref.payload;
      var error = ref.error;

      if (type === actions.START) {
        return Object.assign({}, state, { isLoading: true })
      }
      if (type === actions.SUCCESS) {
        return Object.assign({}, state, {
          isLoading: false,
          data: payload,
          lastSuccess: Date.now(),
          errorTimes: [],
          failedPermanantly: false
        })
      }
      if (type === actions.ERROR) {
        return Object.assign({}, state, {
          isLoading: false,
          errorTimes: state.errorTimes.concat([Date.now()]),
          failedPermanantly: !!(error && error.permanent)
        })
      }
      return state
    }
  };
  result[("select" + ucaseName + "Raw")] = inputSelector;
  result[("select" + ucaseName)] = dataSelector;
  result[("select" + ucaseName + "IsStale")] = isStaleSelector;
  result[("select" + ucaseName + "IsLoading")] = isLoadingSelector;
  result[("select" + ucaseName + "FailedPermanantly")] = failedPermanentlySelector;
  result[("select" + ucaseName + "ShouldUpdate")] = shouldUpdateSelector;
  result[("doFetch" + ucaseName)] = doFetchData

  if (opts.persist) {
    result.persistActions = [ actions.SUCCESS ]
  }

  return result
}

// Thin layer on top of idbkey val with support for versioning and
// max age
var defaultOpts$1 = {maxAge: Infinity, version: 0, lib: idbKeyval}

var getCachedItem = function (key, opts) {
  var ref = Object.assign(opts, defaultOpts$1);
  var maxAge = ref.maxAge;
  var version = ref.version;
  var lib = ref.lib;
  return lib.get(key)
    .then(JSON.parse)
    .then(function (parsed) {
      var age = Date.now() - parsed.time
      if (age > maxAge || version !== parsed.version) {
        lib.delete(key)
        return null
      }
      return {
        age: age,
        data: parsed.data
      }
    })
}

var getAllCached = function (opts) {
  Object.assign(opts, defaultOpts$1)
  var keys
  return opts.lib.keys()
    .then(function (retrivedKeys) {
      keys = retrivedKeys
      return Promise.all(keys.map(function (key) { return getCachedItem(key, opts)
          .then(function (res) { return res.data; }); }
      ))
    })
    .then(function (data) { return data.reduce(function (acc, bundleData, index) {
      if (bundleData) {
        acc[keys[index]] = bundleData
      }
      return acc
    }, {}); })
}

var clearAllCached = function (opts) {
  if ( opts === void 0 ) opts = defaultOpts$1;

  return opts.lib.clear();
}

var cacheItem = function (key, data, opts) {
  Object.assign(opts, defaultOpts$1)
  return opts.lib.set(key, JSON.stringify({
    version: opts.version,
    time: Date.now(),
    data: data
  }))
}

var defaults = { version: 0 }
var caching = function (spec) {
  var opts = Object.assign({}, defaults, spec)

  return {
    name: 'localCache',
    extract: 'persistActions',
    getMiddleware: function (extracted) {
      var combinedActions = {}
      var loop = function ( bundleName ) {
        (extracted[bundleName] || []).forEach(function (type) {
          combinedActions[type] || (combinedActions[type] = [])
          combinedActions[type].push(bundleName)
        })
      };

      for (var bundleName in extracted) loop( bundleName );

      return function (ref) {
        var getState = ref.getState;

        return function (next) { return function (action) {
        var keys = combinedActions[action.type]
        var res = next(action)
        var state = getState()
        if (keys) {
          if (IS_BROWSER) {
            ric(function () {
              Promise.all(keys.map(function (key) { return cacheItem(key, state[key], {version: opts.version}); }
              )).then(function () {
                if (IS_DEBUG) {
                  console.info(("persisted " + (keys.join(', ')) + " because of action " + (action.type)))
                }
              })
            })
          }
        }
        return res
      }; };
      }
    }
  }
}

var raf =
  (IS_BROWSER && require('component-raf')) ||
  (function (func) { setTimeout(func, 0) })

var defaults$1 = {
  idleTimeout: 30000,
  idleAction: 'APP_IDLE',
  doneCallback: null
}

var effects = function (opts) { return ({
  name: 'reactiveDispatch',
  extract: 'effects',
  init: function (store, extracted) {
    opts || (opts = {})
    Object.assign(opts, defaults$1)
    var idleAction = opts.idleAction;
    var idleTimeout = opts.idleTimeout;
    var idleDispatcher = debounce(function () {
      raf(function () { return store.dispatch({type: idleAction}); })
    }, idleTimeout)

    var effectObj = flattenExtractedToObject(extracted)
    for (var item in effectObj) {
      // helpful development errors
      if (process.env.NODE_ENV !== 'production') {
        var actionName = effectObj[item]
        if (!store[item]) {
          throw Error(("Effect key '" + item + "' does not exist on the store. Make sure you're defining as selector by that name."))
        }
        if (!store[actionName]) {
          throw Error(("Effect value '" + actionName + "' does not exist on the store. Make sure you're defining an action creator by that name."))
        }
        if (typeof actionName !== 'string') {
          throw Error(("Effect values must be strings. The effect " + item + " has a value that is: " + (typeof actionName)))
        }
      }
    }

    store.effects = effectObj
    store.activeEffectQueue = []

    var buildActiveEffectQueue = function () {
      for (var selectorName in store.effects) {
        var result = store[selectorName]()
        if (result !== null && store.activeEffectQueue.indexOf(selectorName) === -1) {
          store.activeEffectQueue.push(selectorName)
        }
      }
    }

    var dispatchNext = function () {
      ric(function () {
        var next = store.activeEffectQueue.shift()
        if (next) {
          // make sure it's still relevant
          var result = store[next]()
          if (result !== null) {
            var actionCreatorName = store.effects[next]
            store[actionCreatorName](result)
          }
        }
        if (!IS_BROWSER && !next && (!store.selectAsyncActive || !store.selectAsyncActive())) {
          idleDispatcher.cancel()
          opts.doneCallback && opts.doneCallback()
        }
      })
    }

    var callback = function () {
      buildActiveEffectQueue()
      dispatchNext()
      idleDispatcher()
    }

    store.subscribe(callback)
    callback()
  }
}); }

var getError = function (message, permanent) {
  if ( permanent === void 0 ) permanent = false;

  var err = new Error(message)
  if (permanent) err.permanent = true
  return err
}

var geoErrorArray = [
  'An unknown geolocation error occured',
  'Geolocation permission denied',
  'Geolocation unavailable',
  'Geolocation request timed out'
]

var defaultOpts$2 = {
  timeout: 60000,
  enableHighAccuracy: false,
  persist: true,
  staleAge: 900000, // fifteen minutes
  retryAfter: 60000 // one minute,
}

var geolocation = function (spec) {
  var opts = Object.assign({}, defaultOpts$2, spec)
  return createAsyncResourceBundle$1({
    name: 'geolocation',
    actionBaseType: 'REQUEST_GEOLOCATION',
    getPromise: function () { return new Promise(function (resolve, reject) {
      if (!IS_BROWSER || !navigator.geolocation) {
        reject(getError('Geolocation not supported', true))
      }
      var success = function (position) {
        var res = {}
        var coords = position.coords;
        for (var key in coords) {
          res[key] = coords[key]
        }
        res.timestamp = position.timestamp
        resolve(res)
      }
      var fail = function (ref) {
        var code = ref.code;

        reject(getError(geoErrorArray[code], code === 1))
      }
      var geoOpts = {
        timeout: opts.timeout,
        enableHighAccuracy: opts.enableHighAccuracy
      }
      navigator.geolocation.getCurrentPosition(success, fail, geoOpts)
    }); },
    persist: opts.persist,
    staleAge: opts.staleAge,
    retryAfter: opts.retryAfter
  })
}

var isDefined = function (thing) { return typeof thing !== 'undefined'; }
var ensureString = function (input) { return typeof input === 'string' ? input : qs.stringify(input); }
var IPRE = /^[0-9\.]+$/
var parseSubdomains = function (hostname) {
  if (IPRE.test(hostname)) return []
  return hostname.split('.').slice(0, -2)
}
var removeLeading = function (char, string) { return string.charAt(0) === char ? string.slice(1) : string; }
var ensureLeading = function (char, string) {
  if (string === char || string === '') {
    return ''
  }
  return string.charAt(0) !== char ? char + string : string
}
var loc = (function () {
  if (!IS_BROWSER) return {}
  return window.location || self.location
})()
var defaults$2 = {
  name: 'url',
  inert: !IS_BROWSER,
  actionType: 'UPDATE_URL'
}

var url = function (opts) {
  var config = Object.assign({}, defaults$2, opts)
  var actionType = config.actionType

  var selectUrlRaw = function (state) { return state[config.name]; }
  var selectUrlObject = createSelector.createSelector(selectUrlRaw, function (urlState) { return new URL(urlState.url, true); })
  var selectQueryObject = createSelector.createSelector(selectUrlObject, function (urlObj) { return urlObj.query; })
  var selectQueryString = createSelector.createSelector(selectQueryObject, function (queryObj) { return qs.stringify(queryObj); })
  var selectPathname = createSelector.createSelector(selectUrlObject, function (urlObj) { return urlObj.pathname; })
  var selectHash = createSelector.createSelector(selectUrlObject, function (urlObj) { return removeLeading('#', urlObj.hash); })
  var selectHashObject = createSelector.createSelector(selectHash, function (hash) { return qs.parse(hash); })
  var selectSubdomains = createSelector.createSelector(selectUrlObject, function (urlObj) { return parseSubdomains(urlObj.hostname); })

  var doUpdateUrl = function (newState, opts) {
    if ( opts === void 0 ) opts = {replace: false};

    return function (ref) {
    var dispatch = ref.dispatch;
    var getState = ref.getState;

    var state = (typeof newState === 'string') ? { pathname: newState, hash: '', query: '' } : newState
    var url = new URL(selectUrlRaw(getState()).url, true)
    if (isDefined(state.pathname)) url.set('pathname', state.pathname)
    if (isDefined(state.hash)) url.set('hash', ensureString(state.hash))
    if (isDefined(state.query)) url.set('query', ensureString(state.query))
    dispatch({ type: actionType, payload: { url: url.href, replace: opts.replace } })
  };
  }
  var doReplaceUrl = function (url) { return doUpdateUrl(url, {replace: true}); }
  var doUpdateQuery = function (query, opts) {
      if ( opts === void 0 ) opts = {replace: true};

      return doUpdateUrl({ query: ensureString(query) }, opts);
  }
  var doUpdateHash = function (hash, opts) {
      if ( opts === void 0 ) opts = {replace: false};

      return doUpdateUrl({ hash: ensureLeading('#', ensureString(hash)) }, opts);
  }

  return {
    name: config.name,
    init: function (store) {
      if (config.inert) {
        return
      }

      var lastState = store.selectUrlRaw()

      var setCurrentUrl = function () {
        store.doUpdateUrl({
          pathname: loc.pathname,
          hash: loc.hash,
          query: loc.search
        })
      }

      window.addEventListener('popstate', setCurrentUrl)

      store.subscribe(function () {
        var newState = store.selectUrlRaw()
        var newUrl = newState.url
        if (lastState !== newState && newUrl !== loc.href) {
          try {
            window.history[newState.replace ? 'replaceState' : 'pushState']({}, null, newState.url)
          } catch (e) {
            console.error(e)
          }
        }
        lastState = newState
      })

      window.addEventListener('popstate', setCurrentUrl)
    },
    getReducer: function () {
      var initialState = {
        url: !config.inert && IS_BROWSER
          ? loc.href
          : '/',
        replace: false
      }

      return function (state, ref) {
        if ( state === void 0 ) state = initialState;
        var type = ref.type;
        var payload = ref.payload;

        if (type === '@@redux/INIT' && typeof state === 'string') {
          return {
            url: state,
            replace: false
          }
        }
        if (type === actionType) {
          return Object.assign({
            url: payload.url || payload,
            replace: !!payload.replace
          })
        }
        return state
      }
    },
    doUpdateUrl: doUpdateUrl,
    doReplaceUrl: doReplaceUrl,
    doUpdateQuery: doUpdateQuery,
    doUpdateHash: doUpdateHash,
    selectUrlRaw: selectUrlRaw,
    selectUrlObject: selectUrlObject,
    selectQueryObject: selectQueryObject,
    selectQueryString: selectQueryString,
    selectPathname: selectPathname,
    selectHash: selectHash,
    selectHashObject: selectHashObject,
    selectSubdomains: selectSubdomains
  }
}

var appTimeBundle = appTimeBundle$1
var asyncCountBundle = asyncCount
var inspectBundle = inspect
var cachingBundle = caching
var createRouteBundle = createRoutingBundle
var createAsyncResourceBundle = createAsyncResourceBundle$1
var effectsBundle = effects
var urlBundle = url
var composeBundlesRaw = composeBundles$1
var geolocationBundle = geolocation
var composeBundles = function () {
  var userBundles = [], len = arguments.length;
  while ( len-- ) userBundles[ len ] = arguments[ len ];

  userBundles || (userBundles = [])
  var bundles = [
    appTimeBundle$1,
    asyncCount,
    url(),
    effects(),
    caching(),
    inspect ].concat( userBundles
  )
  return composeBundles$1.apply(void 0, bundles)
}

exports.appTimeBundle = appTimeBundle;
exports.asyncCountBundle = asyncCountBundle;
exports.inspectBundle = inspectBundle;
exports.cachingBundle = cachingBundle;
exports.createRouteBundle = createRouteBundle;
exports.createAsyncResourceBundle = createAsyncResourceBundle;
exports.effectsBundle = effectsBundle;
exports.urlBundle = urlBundle;
exports.composeBundlesRaw = composeBundlesRaw;
exports.geolocationBundle = geolocationBundle;
exports.composeBundles = composeBundles;
exports.createSelector = createSelector.createSelector;
exports.resolveSelectors = createSelector.resolveSelectors;
exports.IS_BROWSER = IS_BROWSER;
exports.IS_DEBUG = IS_DEBUG;
exports.flattenExtractedToObject = flattenExtractedToObject;
exports.flattenExtractedToArray = flattenExtractedToArray;
exports.getCachedItem = getCachedItem;
exports.getAllCached = getAllCached;
exports.clearAllCached = clearAllCached;
exports.cacheItem = cacheItem;
